"""
Math library tests for `calc.py` library

Run in /src directory:
    make test
'pytest' module has to be preinstalled
"""

import pytest
from calculator.calc import *


def test_empty_string():
    result = EvaluateString("")
    assert result == 0


def test_non_empty_string():
    assert EvaluateString("5") == 5
    assert EvaluateString("-8") == -8
    assert EvaluateString("12.98") == 12.98
    assert EvaluateString("21 + 8") == 29
    assert EvaluateString("23 -  2") == 21
    assert EvaluateString(" 4*3") == 12
    assert EvaluateString("88/8 ") == 11


def test_positive_integer_addition():
    assert Add(16, 9) == 25
    assert Add(10, 200) == 210
    assert Add(525, 475) == 1000


def test_negative_integer_addition():
    assert Add(-552, 38) == -514
    assert Add(-10, 10) == 0
    assert Add(-2, 18) == 16


def test_float_addition():
    assert Add(0.1, 1.15) == 1.25
    assert Add(0.1, 0.2) == 0.3
    assert Add(-12.6, 5.4) == -7.2


def test_positive_substraction():
    assert Subtract(95, 21) == 74
    assert Subtract(860, 114) == 746
    assert Subtract(62, 62) == 0


def test_negative_subtraction():
    assert Subtract(-36, 6) == -42
    assert Subtract(515, 6899) == -6384
    assert Subtract(0, -8) == 8


def test_positive_multiplication():
    assert Multiply(6, 3) == 18
    assert Multiply(0, 5) == 0
    assert Multiply(12, 12) == 144


def test_negative_multiplication():
    assert Multiply(-6, 2) == -12
    assert Multiply(9, -4) == -36
    assert Multiply(-140, -11) == 1540


def test_float_multiplication():
    assert Multiply(5.5, 3.3) == 18.15
    assert Multiply(-3.0, 15) == -45
    assert Multiply(1.1, -3) == -3.3


def test_positive_division():
    assert Divide(6, 3) == 2
    assert Divide(0, 5) == 0
    assert Divide(110, 11) == 10


def test_divide_by_zero():
    with pytest.raises(ZeroDivisionError):
        Divide(13, 0)


def test_negative_division():
    assert Divide(-9, 3) == -3
    assert Divide(28, -4) == -7
    assert Divide(-120, -40) == 3


def test_float_division():
    assert Divide(3.5, 5) == 0.7
    assert Divide(2.5, 0.5) == 5
    assert Divide(3.3, 1.1) == 3


def test_integer_factorial():
    assert Factorial(5) == 120
    assert Factorial(8) == 40320
    assert Factorial(1) == 1


def test_zero_factorial():
    assert Factorial(0) == 1


def test_negative_factorial():
    with pytest.raises(ValueError):
        Factorial(-5)


def test_float_factorial():
    with pytest.raises(ValueError):
        Factorial(-6.5)


def test_simple_addition_in_expression():
    assert EvaluateString("1 + 1") == 2


def test_combined_operations():
    assert EvaluateString("2 + 3 * 4") == 14


def test_function_combinations():
    assert EvaluateString("sin(30) + cos(60)") == Sine(30) + Cosine(60)


def test_nested_functions():
    assert EvaluateString("log (sin(45) + cos(45))") == Logarithm(Sine(45) + Cosine(45))


def test_division_by_zero_in_expression():
    with pytest.raises(ZeroDivisionError):
        EvaluateString("5 / 0")


def test_factorial_of_non_integer():
    with pytest.raises(ValueError):
        EvaluateString("5.5!")


def test_root_of_negative_number():
    with pytest.raises(ValueError):
        EvaluateString("sqrt(-16)")


def test_invalid_input():
    with pytest.raises(ValueError):
        EvaluateString("abc + 5")


def test_exponentiation():
    assert EvaluateString("3 ^ 2") == 9


def test_complex_expression():
    assert EvaluateString("3 + 5 * (2 ^ 3) - 7 / sin(30)") == 3 + 5 * (2 ** 3) - 7 / Sine(30)


def test_negative_numbers_handling():
    assert EvaluateString("-5 - -3") == Subtract(-5, -3)


def test_invalid_function_usage():
    with pytest.raises(ValueError):
        EvaluateString("sin cos 30")

# End of tests
